import asyncio
import json

from attrdictionary import AttrDict
import aiohttp

from misspy.hook import hook
from . import exception


class MiWS:
    def __init__(self, address, i, ssl=True) -> None:
        self.i = i
        self.address = address
        self.ssl = ssl

    async def ws_handler(self):
        try:
            procotol = "ws://"
            if self.ssl:
                procotol = "wss://"
            session = aiohttp.ClientSession()
            async with session.ws_connect(
                f"{procotol}{self.address}/streaming?i={self.i}"
            ) as self.connection:
                try:
                    await hook.functions["ready"]()
                except KeyError:
                    pass
                while True:
                    try:
                        recv = await self.connection.receive_json()
                    except AttributeError:
                        pass
                    try:
                        if recv["type"] == "channel":
                            if recv["body"]["type"] == "note":
                                await hook.functions[recv["body"]["type"]](
                                    AttrDict(recv["body"]["body"])
                                )
                            elif recv["body"]["type"] == "notification":
                                await hook.functions[recv["body"]["body"]["type"]](
                                    AttrDict(recv["body"]["body"])
                                )
                            elif recv["body"]["type"] == "follow":
                                await hook.functions[recv["body"]["type"]](
                                    AttrDict(recv["body"]["body"])
                                )
                            elif recv["body"]["type"] == "followed":
                                await hook.functions[recv["body"]["type"]](
                                    AttrDict(recv["body"]["body"])
                                )
                            elif recv["type"] == "noteUpdated":
                                await hook.functions[recv["body"]["type"]](
                                    AttrDict(recv["body"])
                                )
                            else:
                                await hook.functions[recv["body"]["type"]](
                                    AttrDict(recv["body"])
                                )
                    except KeyError:
                        pass
        except Exception as e:
            raise exception.WebsocketError(e)



import json
import requests

import aiohttp
import httpx

from .exception import HTTPException, ClientException, RateLimitError

h_t = {"Content-Type": "application/json"}

def request_sync(
    address,
    i=None,
    endpoint="test",
    jobj: dict = {"required": True},
    header: dict = h_t,
    ssl: bool=True
):
    """request_sync (internal function)

    Args:
        address (string): instance address
        i (string): user token
        endpoint (string): endpoint (example: notes/create)
        jobj (dict): request params. Do not include the i. Defaults to {"Content-Type": "application/json"}.
        header (dict, optional): request header. Defaults to {"Content-Type": "application/json"}.

    Returns:
        dict: request result
    """
    url = address + "/api/" + endpoint
    if i is not None:
        jobj["i"] = i
    res = requests.post(url, data=json.dumps(jobj, ensure_ascii=False), headers=header, verify=ssl)
    try:
        return res.json()
    except:
        return True


async def request(
    address,
    i=None,
    endpoint="ping",
    jobj: dict = {},
    files=None,
    header=h_t,
):
    """request (internal function)

    Args:
        address (string): instance address
        i (string): user token
        endpoint (string): endpoint (example: notes/create)
        jobj (dict): request params. Do not include the i.

    Returns:
        dict: request result
    """
    async with aiohttp.ClientSession() as client:
        url = address + "/api/" + endpoint
        if i is not None:
            jobj["i"] = i
        if files is not None:
            async with httpx.AsyncClient() as client:
                if header is not None:
                    res = await client.post(
                        url,
                        data=jobj,
                        files=files,
                        headers=header,
                    )
                else:
                    res = await client.post(url, files=files, data=jobj)
            try:
                resp = res.json()
            except json.JSONDecodeError:
                resp = json.loads(res.text)
        else:
            if header is not None:
                res = await client.post(
                    url, data=json.dumps(jobj, ensure_ascii=False), headers=header
                )
                try:
                    return await res.json()
                except json.JSONDecodeError:
                    return True
            else:
                res = await client.post(url, data=json.dumps(jobj, ensure_ascii=False))
            try:
                resp = await res.json()
            except json.JSONDecodeError:
                resp = json.loads(await res.text)
        if resp.get("error").get("kind") is not None:
            if resp["error"]["code"] == "RATE_LIMIT_EXCEEDED":
                raise RateLimitError("We are being rate limited. Please try again in a few moments.")
            raise ClientException(
                resp["error"]["message"]
                + "\nid: "
                + resp["error"]["id"]
            )
        else:
            print(type(resp))
            return resp


class MisskeyException(Exception):
    """Base exception class for misspy

    Ideally, this should be caught and the exception handled.
    """

    pass


class WebsocketError(MisskeyException):
    """websocket connection error class for misspy

    Ideally, this should be caught and the exception handled.
    """

    pass


class ClientException(MisskeyException):
    """Parent class of error classes such as login failure."""

class RateLimitError(ClientException):
    """RateLimitError trigger: RATE_LIMIT_EXCEEDED"""

class AuthenticationFailed(ClientException):
    """Class called if MisskeyAPI authentication fails."""


class MiAuthFailed(AuthenticationFailed):
    """Class called if MiAuth fails."""


class HTTPException(MisskeyException):
    """Error class called when a request to the misskey server's API fails.

    Called when httpx.HTTPError is caught.
    """
